package de.andrena.springbootadvancedtesting.booking;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.cache.CacheManager;

import de.andrena.springbootadvancedtesting.swapiclient.StarwarsApiClient;
import de.andrena.springbootadvancedtesting.swapiclient.StarwarsStarship;

@SpringBootTest
class StarshipBookingServiceTest {

	@Autowired
	private StarshipBookingService bookingService;

	@Autowired
	private CacheManager cacheManager;

	@MockBean
	private StarwarsApiClient starwarsApiClient;

	@AfterEach
	void clearCache() {
		cacheManager.getCache("bookable-starships").clear();
	}

	@Test
	void returnsStarshipsFromStarwarsApiClient() {
		List<StarwarsStarship> twoStarships = List.of( //
				new StarwarsStarship("A-wing"), //
				new StarwarsStarship("Jedi starfighter"));

		when(starwarsApiClient.findAll(any())).thenReturn(twoStarships);

		List<Starship> result = bookingService.findBookableStarships(0);

		assertThat(result).hasSize(2) //
				.extracting(Starship::getName).containsExactly("A-wing", "Jedi starfighter");

		verify(starwarsApiClient).findAll(any());
	}

	@Test
	void cachesStarshipsFromStarwarsApiClient() {
		when(starwarsApiClient.findAll(any())).thenReturn(List.of(new StarwarsStarship(), new StarwarsStarship()));

		assertThat(bookingService.findBookableStarships(0)).hasSize(2);
		assertThat(bookingService.findBookableStarships(0)).hasSize(2);
		assertThat(bookingService.findBookableStarships(0)).hasSize(2);

		verify(starwarsApiClient, times(1)).findAll(any());
	}

}
